/*
 * UART.c
 *
 *  Created on: Sep 13, 2019
 *      Author: a0222913
 */

#include "derivative.h" /* include peripheral declarations SSKEAZ64M4 */
#include "BeaconTorch.h"

extern char timeOutFlag;
extern char responseData[]; //store the response data bytes
extern unsigned int receiveByteNum; //store the received byte number

void uartInitial()
{
	/*PTD7 -- UART2_TXD*/
	/*PTD6 -- UART2_RXD*/
	GPIOA_PDDR |= 1<<31; /*set PTD7 as output*/
	GPIOA_PDDR &=~(1<<30); /*set PTD6 as intput*/
	PORT_PUE0 &=~(1<<30);/* disable PTD6 pull up*/
	GPIOA_PIDR &=~(1<<30);/*PTD6 Pin is configured for General Purpose Input, provided the pin is configured for any digital function*/
	SIM_SCGC |= SIM_SCGC_UART2_MASK; /*Enable bus clock in UART2*/
	UART2_BDH = 0x00;
	UART2_BDL = 2;/*Buard rate = 6000K/16*2 = 185Kbps*/
	UART2_C1 = 0x00;/*Normal - start + 8 data bits (lsb first) + stop*/
	UART2_C2 |= UART_C2_TE_MASK + UART_C2_RE_MASK; /*Enable Transmitter and Receiver*/
	Enable_Interrupt(UART2_IRQn);/*Enable UART0 interrupt*/
}

//void uartInitial()
//{
//	/*PTC7 -- UART1_TXD*/
//	/*PTC6 -- UART1_RXD*/
//	GPIOA_PDDR |= 1<<23; /*set PTC7 as output*/
//	GPIOA_PDDR &=~(1<<22); /*set PTC6 as intput*/
//	PORT_PUE0 &=~(1<<22);/* diable PTC6 pull up*/
//	GPIOA_PIDR &=~(1<<22);/*PTC6 Pin is configured for General Purpose Input, provided the pin is configured for any digital function*/
//	SIM_SCGC |= SIM_SCGC_UART1_MASK; /*Enable bus clock in UART1*/
//	UART1_BDH = 0x00;
//	UART1_BDL = 1;/*Buard rate = 6000K/16 = 375Kbps*/
//	UART1_C1 = 0x00;/*Normal - start + 8 data bits (lsb first) + stop*/
//	UART1_C2 |= UART_C2_TE_MASK + UART_C2_RE_MASK; /*Enable Transmitter and Receiver*/
//	Enable_Interrupt(UART1_IRQn);/*Enable UART0 interrupt*/
//}

void uartWrite(char commandFrame[], unsigned int frameLength, unsigned int checkResponse, unsigned int responseLength)
{
	unsigned int i;
	//if(checkResponse==TRUE)
	//{
	//	receiveByteNum=0;
	//	UART2_C2 |= UART_C2_RIE_MASK;/*Enable receiver interrupt for RDRF*/
	//}
	for(i=0;i<frameLength;i++)
	{
		while(!(UART2_S1 & UART_S1_TDRE_MASK)); //wait until transmit data register is empty
		UART2_D = commandFrame[i]; //send one data byte to uart data register
	}
	if(checkResponse == TRUE)
	{
		timeOut(10); //launch timer and set wait time = 200us
						// This time should be larger than the time to receive all the response bytes,
						// And the response receiving time depends on the buard rate and the number of response byte,
						// For example, baurd rate = 115200, 2 response byte, so the wait time should be larger than 2*10*1/115200 = 2.083ms
						// Why 2*10 because for each byte there are additional 1 start bit and 1 stop bit
		while(!((timeOutFlag == 1) || (receiveByteNum == responseLength))); //received all response byte or the wait time exceeds the specified time

		if(receiveByteNum<responseLength)
		{
			// You can take some action once the response has not been received
									//Below is an example:
			//P1OUT |= BIT0; //set P1.0 high to turn on the red LED in the launch pad
		}
		else
		{
			//P1OUT &= ~BIT0; //successfully received response, clear the red LED
		}
		UART2_C2 &= ~UART_C2_RIE_MASK;/*Disable receiver interrupt for RDRF*/
		FTM1_SC &= ~FTM_SC_TOIE_MASK;/*disable FTM1 TOF interrupt*/
		FTM1_SC &= ~FTM_SC_TOF_MASK;/* Read to clear flag */
		FTM1_SC &= ~FTM_SC_CLKS_MASK;/*disable FTM1*/
	}
}

void UART2_IRQHandler()/*UART0 interrupt function*/
	{
		UART2_S1 & UART_S1_RDRF_MASK; /*clear receive data register full flag*/
		responseData[receiveByteNum++] = UART2_D; //store response to array
	}
